/* -*- c++ -*- */
/*
 * Copyright 2004 Free Software Foundation, Inc.
 * 
 * This file is part of GNU Radio
 * 
 * GNU Radio is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * GNU Radio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GNU Radio; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

/*
 * config.h is generated by configure.  It contains the results
 * of probing for features, options etc.  It should be the first
 * file included in your .cc file.
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#include <gnuradio/io_signature.h>
#include "estimate_sample_rate_bf_impl.h"

namespace gr {
  namespace dab {

estimate_sample_rate_bf::sptr
estimate_sample_rate_bf::make(float expected_sample_rate, int frame_length)
{
  return gnuradio::get_initial_sptr
    (new estimate_sample_rate_bf_impl(expected_sample_rate, frame_length));
}

/*
 * Specify constraints on number of input and output streams.
 * This info is used to construct the input and output signatures
 * (2nd & 3rd args to gr_block's constructor).  The input and
 * output signatures are used by the runtime system to
 * check that a valid number and type of inputs and outputs
 * are connected to this block.  In this case, we accept
 * only 1 input and 1 output.
 */
static const int MIN_IN = 1;  // mininum number of input streams
static const int MAX_IN = 1;  // maximum number of input streams
static const int MIN_OUT = 1;  // minimum number of output streams
static const int MAX_OUT = 1;  // maximum number of output streams

estimate_sample_rate_bf_impl::estimate_sample_rate_bf_impl(float expected_sample_rate, int frame_length)
  : gr::sync_block("estimate_sample_rate_bf",
       gr::io_signature::make (MIN_IN, MAX_IN, sizeof (char)),
       gr::io_signature::make (MIN_OUT, MAX_OUT, sizeof (float))),
    d_zeros(0), d_expected_sample_rate(expected_sample_rate), d_real_sample_rate(expected_sample_rate), d_found_first_frame(0), d_frame_length(frame_length)
{
}

/*
 * Our virtual destructor.
 */
estimate_sample_rate_bf_impl::~estimate_sample_rate_bf_impl()
{
  // nothing else required in this example
}

int 
estimate_sample_rate_bf_impl::work(int noutput_items,
      gr_vector_const_void_star &input_items,
      gr_vector_void_star &output_items)
{
  const char *in = (const char *) input_items[0];
  float *out = (float *) output_items[0];

  int i=0;
  if (d_found_first_frame) {
    for(i=0; i<noutput_items; i++) {
      d_zeros++;
      if (*in == 1) {
        d_real_sample_rate = (float)d_zeros / (float)d_frame_length * d_expected_sample_rate;
        // fprintf(stderr,"estimated sample rate: %f\n", d_real_sample_rate);
        d_zeros = 0;
      }
      in++;
      *out++ = d_real_sample_rate;
    } 
  } else {
    while (i<noutput_items && d_found_first_frame==0) {
      i++;
      if (*in == 1)
        d_found_first_frame = 1;
      in++;
      *out++ = d_real_sample_rate;
    }
  }

  // Tell runtime system how many output items we produced.
  return i;
}

}
}
